import { AbortController } from "./utils.ts";

type LetterUrls = [normal: string, textured: string];
type Letter = LetterUrls | { variants: Partial<LetterUrls>[] };

const b = ([fname]: TemplateStringsArray) =>
  `https://nicimesto.cz/wp-content/uploads/2024/03/${fname}.png`;

export const letterMap = {
  A: [b`A`, b`A-2`],
  Á: [b`A-1`, b`A-3`],
  B: [b`B`, b`B-1`],
  C: [b`C`, b`C-3`],
  Ć: [b`C-1`, b`C-4`],
  Č: [b`C-2`, b`C-5`],
  CH: { variants: [[b`CH`, b`CH-1`], [b`CH2`]] },
  D: [b`D`, b`D-2`],
  Ď: [b`D-1`, b`D-3`],
  E: [b`E`, b`E-2`],
  Ě: [b`E-1`, b`E-3`],
  F: {
    variants: [
      [b`F`, b`F-1`],
      [undefined, b`F2`],
    ],
  },
  G: [b`G`, b`G-1`],
  H: [b`H`, b`H-1`],
  I: [b`I`, b`I-2`],
  Í: [b`I-1`, b`I-3`],
  J: [b`J`, b`J-1`],
  K: [b`K`, b`K-1`],
  L: {
    variants: [
      [b`L`, b`L-2`],
      [b`L2`, b`L2-1`],
      [b`L3`, b`L3-1`],
    ],
  },
  Ľ: [b`L-1`, b`L-3`],
  M: [b`M`, b`M-1`],
  N: [b`N`, b`N-2`],
  Ň: [b`N-1`, b`N-2`],
  O: [b`O`, b`O-3`],
  Ó: {
    variants: [
      [b`O-1`, b`O-4`],
      [undefined, b`O2`],
    ],
  },
  Ô: [b`O-2`, b`O-5`],
  P: [b`P`, b`P-1`],
  Q: {
    variants: [
      [b`Q`, b`Q-1`],
      [undefined, b`Q2`],
    ],
  },
  R: [b`R`, b`R-2`],
  Ř: [b`R-1`, b`R-3`],
  S: [b`S`, b`S-2`],
  Š: [b`S-1`, b`S-3`],
  T: [b`T`, b`T`], // chybí texturka varianta
  Ť: [b`T-1`, b`T-2`],
  U: [b`U`, "https://nicimesto.cz/wp-content/uploads/2024/11/U-1.png"],
  Ů: [b`U-1`, b`U-3`],
  Ü: [b`UU`, b`UU-1`],
  V: [b`V`, b`V-1`],
  W: [b`W`, b`W-1`],
  X: [b`X`, b`X-1`],
  Y: [b`Y`, b`Y-1`],
  Z: [b`Z`, b`Z-2`],
  Ž: [b`Z-1`, b`Z-3`],
} satisfies Record<string, Letter>;

export const symbolMap = {
  "❓️": [b`unnamed-file-1`, b`unnamed-file-5`],
  "⁉️": [b`unnamed-file`, b`unnamed-file-4`],
  "❗️": [b`unnamed-file-2`],
  "🐾": [b`TLAPA-1`, b`Tlap`],
  "#️⃣": [b`unnamed-file-3`],
  ":": [
    "https://nicimesto.cz/wp-content/uploads/2025/01/Artboard2_12-scaled.jpg",
  ],
  "–": [
    "https://nicimesto.cz/wp-content/uploads/2025/01/Artboard2_2-scaled.jpg",
  ],
  "=": ["https://nicimesto.cz/wp-content/uploads/2025/01/Artboard2-scaled.jpg"],
  "≠": ["https://nicimesto.cz/wp-content/uploads/2025/01/Artboard1-scaled.jpg"],
  "1": [
    "https://nicimesto.cz/wp-content/uploads/2025/01/Artboard2_3-scaled.jpg",
  ],
  "2": [
    "https://nicimesto.cz/wp-content/uploads/2025/01/Artboard2_4-scaled.jpg",
  ],
  "3": [
    "https://nicimesto.cz/wp-content/uploads/2025/01/Artboard2_5-scaled.jpg",
  ],
  "4": [
    "https://nicimesto.cz/wp-content/uploads/2025/01/Artboard2_6-scaled.jpg",
  ],
  "5": [
    "https://nicimesto.cz/wp-content/uploads/2025/01/Artboard2_7-scaled.jpg",
  ],
  "6": [
    "https://nicimesto.cz/wp-content/uploads/2025/01/Artboard2_8-scaled.jpg",
  ],
  "7": [
    "https://nicimesto.cz/wp-content/uploads/2025/01/Artboard2_9-scaled.jpg",
  ],
  "8": [
    "https://nicimesto.cz/wp-content/uploads/2025/01/Artboard2_10-scaled.jpg",
  ],
  "9": [
    "https://nicimesto.cz/wp-content/uploads/2024/03/9.png",
    "https://nicimesto.cz/wp-content/uploads/2025/01/Artboard2_11-scaled.jpg",
  ],
  "𝄢": [b`BASS`],
} satisfies Record<string, string[]>;

export const validateGlyphUrl = async (url: string) => {
  const { signal, abort } = new AbortController();
  const { ok, status, statusText, headers } = await fetch(url, { signal });
  const mime = headers.get("Content-Type");
  if (!ok) throw `HTTP ${status}: ${statusText}; URL: ${url}`;
  if (mime !== "image/png")
    throw `Unexpected Content-Type: ${mime}; URL: ${url}`;
  abort();
};

export const validateAllGlyphs = async () => {
  for (const v of Object.values(letterMap)) {
    const glyphsOrUndef = "variants" in v ? v.variants.flat() : v;
    const glyphs = glyphsOrUndef.filter((g) => g !== undefined);
    for (const g of glyphs) await validateGlyphUrl(g);
  }
};

export type Glyph = keyof typeof letterMap | keyof typeof symbolMap;
export type Font = "normal" | "textured";
export type GlyphVariant = [Glyph, number];

const variantsOfGlyph = (glyph: Glyph, font: Font): string[] => {
  if (glyph in symbolMap) {
    return symbolMap[glyph as keyof typeof symbolMap];
  }

  const fontIndex = font === "normal" ? 0 : 1;
  const v = letterMap[glyph as keyof typeof letterMap];
  const glyphsByFont: Partial<LetterUrls>[] =
    "variants" in v ? v.variants : [v];

  return glyphsByFont.map((g) => g[fontIndex]).filter((g) => g !== undefined);
};

export const countGlyphVariants = (glyph: Glyph, font: Font): number => {
  return variantsOfGlyph(glyph, font).length;
};

export const glyphVariantToUrl = (
  glyphVariant: GlyphVariant,
  font: Font
): string => {
  const [glyph, variant] = glyphVariant;
  return variantsOfGlyph(glyph, font)[variant];
};
